using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using AdkNet4Wrapper;

namespace DemoApiCSharp
{
    public partial class DemoProjektMainForm : Form
    {
        private int pData;
        private int pData2;
        private Api.ADKERROR error;
        private bool bShowed;

        public DemoProjektMainForm()
        {
            InitializeComponent();
            pData = 0;
            pData2 = 0;
            ChangeEnable(false);
            KUNDLISTA.FullRowSelect = true;
            FKT_LIST.FullRowSelect = true;
            bShowed = false;
        }

        private void ChangeEnable(bool enable)
        {
            KUND_BN_ADD.Enabled = enable;
            KUND_BN_DELETE.Enabled = enable;
            KUND_BN_FIRST.Enabled = enable;
            KUND_BN_NEXT.Enabled = enable;
            KUND_BN_SHOW.Enabled = enable;
            KUND_BN_UPDATE.Enabled = enable;
            KFKT_BN_NEW.Enabled = enable;
            KFKT_BN_LIST.Enabled = enable;
            KFKT_BN_DEL.Enabled = enable;
        }

        private void OpenFtg_Click(object sender, EventArgs e)
        {
            PathSettings ps = new PathSettings();
            
            if (ps.ShowDialog(this) != DialogResult.Cancel)
            {
                String SysPath = ps.psys;
                String SysFtg = ps.pftg;

                ps.Dispose();
                Api.ADKERROR error;
                error.lRc = Api.ADKE_OK;

                String UserName = "";
                String Password = "";
                do
                {
                    if (error.lRc == Api.ADKE_LOGIN_FAIL)
                    {
                        LoginForm loginForm = new LoginForm();
                        if (loginForm.ShowDialog(this) == DialogResult.OK)
                        {
                            UserName = loginForm.TxtUserName.Text;
                            Password = loginForm.TxtPassword.Text;
                        }
                        else
                        {
                            return;
                        }
                    }

                    pData = pData2 = 0;
                    error = Api.AdkOpen2(SysPath, SysFtg, UserName, Password);
                }
                while (error.lRc == Api.ADKE_LOGIN_FAIL);

                if (error.lRc != 0)
                {
                    String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                    MessageBox.Show(errortext);
                    pData = 0;
                }
                else
                {
                    pData = Api.AdkCreateData(Api.ADK_DB_CUSTOMER);
                    pData2 = Api.AdkCreateData(Api.ADK_DB_INVOICE_HEAD);
                    ChangeEnable(true);
                }
            }
        }

        private void CloseFtg_Click(object sender, EventArgs e)
        {
            Api.AdkClose();
            ChangeEnable(false);
        }

        private void avslutaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void ShowKundList()
        {
            KUNDLISTA.Items.Clear();
            error = Api.AdkFirst(pData);
            if (error.lRc != Api.ADKE_OK)
            {
                String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                MessageBox.Show(errortext);
            }

            int iX = 0;
            while (error.lRc != Api.ADKE_EOF)
            {
                Api.AdkGetStr(pData, Api.ADK_CUSTOMER_NUMBER, out var KundNummer);
                Api.AdkGetStr(pData, Api.ADK_CUSTOMER_NAME, out var KundNamn);
                Api.AdkGetStr(pData, Api.ADK_CUSTOMER_TELEPHONE, out var KundTel);

                ListViewItem item = new ListViewItem(KundNummer, iX);
                item.SubItems.Add(KundNamn);
                item.SubItems.Add(KundTel);

                KUNDLISTA.Items.Add(item);

                error = Api.AdkNext(pData);
                iX++;
            }
        }

        private void KUND_BN_SHOW_Click(object sender, EventArgs e)
        {
            this.ShowKundList();
            bShowed = true;
        }

        private void SetDataToFieldKund()
        {
            String info;
            Api.AdkGetStr(pData, Api.ADK_CUSTOMER_NUMBER, out info);
            KUND_KUNDNR_BOX.Text = info;

            Api.AdkGetStr(pData, Api.ADK_CUSTOMER_NAME, out info);
            KUND_NAMN_BOX.Text = info;

            Api.AdkGetStr(pData, Api.ADK_CUSTOMER_ORGANISATION_NUMBER, out info);
            KUND_ORGNR_BOX.Text = info;

            Api.AdkGetStr(pData, Api.ADK_CUSTOMER_TELEPHONE, out info);
            KUND_TELEFON_BOX.Text = info;

            Api.AdkGetStr(pData, Api.ADK_CUSTOMER_SHORT_NAME, out info);
            KUND_KORTNAMN_BOX.Text = info;

            Boolean check;
            Api.AdkGetBool(pData, Api.ADK_CUSTOMER_EU_CUSTOMER, out check);
            KUND_EU.Checked = check;

            Api.AdkGetBool(pData, Api.ADK_CUSTOMER_EXPORT, out check);
            KUND_EXPORT.Checked = check;
        }

        private void GetDataFromFieldKund()
        {
            Api.AdkSetStr(pData, Api.ADK_CUSTOMER_NUMBER, KUND_KUNDNR_BOX.Text);

            Api.AdkSetStr(pData, Api.ADK_CUSTOMER_NAME, KUND_NAMN_BOX.Text);

            Api.AdkSetStr(pData, Api.ADK_CUSTOMER_ORGANISATION_NUMBER, KUND_ORGNR_BOX.Text);

            Api.AdkSetStr(pData, Api.ADK_CUSTOMER_TELEPHONE, KUND_TELEFON_BOX.Text);

            Api.AdkSetStr(pData, Api.ADK_CUSTOMER_SHORT_NAME, KUND_KORTNAMN_BOX.Text);

            Api.AdkSetBool(pData, Api.ADK_CUSTOMER_EU_CUSTOMER, KUND_EU.Checked);

            Api.AdkSetBool(pData, Api.ADK_CUSTOMER_EXPORT, KUND_EXPORT.Checked);
        }

        private void KUNDLISTA_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (KUNDLISTA.FocusedItem != null)
            {
                Api.AdkSetStr(pData, Api.ADK_CUSTOMER_NUMBER, KUNDLISTA.FocusedItem.Text);
                error = Api.AdkFind2(pData, false);
                if (error.lRc != Api.ADKE_OK)
                {
                    String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                    MessageBox.Show(errortext);
                }

                SetDataToFieldKund();
            }
        }

        private void KUND_BN_NEXT_Click(object sender, EventArgs e)
        {
            error = Api.AdkNext(pData);
            if (error.lRc != Api.ADKE_OK)
            {
                String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                MessageBox.Show(errortext);
            }
            else
            {
                SetDataToFieldKund();
            }
        }

        private void KUND_BN_FIRST_Click(object sender, EventArgs e)
        {
            error = Api.AdkFirst(pData);
            if (error.lRc != Api.ADKE_OK)
            {
                String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                MessageBox.Show(errortext);
            }
            else
            {
                SetDataToFieldKund();
            }
        }

        private void KUND_BN_ADD_Click(object sender, EventArgs e)
        {
            GetDataFromFieldKund();
            error = Api.AdkAdd(pData);
            if (error.lRc != Api.ADKE_OK)
            {
                String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                MessageBox.Show(errortext);
            }
            else
            {
                if (bShowed)
                {
                    this.ShowKundList();
                }
            }
        }

        private void KUND_BN_UPDATE_Click(object sender, EventArgs e)
        {
            GetDataFromFieldKund();
            error = Api.AdkUpdate(pData);
            if (error.lRc != Api.ADKE_OK)
            {
                String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                MessageBox.Show(errortext);
            }
            else
            {
                if (bShowed)
                {
                    this.ShowKundList();
                }
            }
        }

        private void KUND_BN_DELETE_Click(object sender, EventArgs e)
        {
            GetDataFromFieldKund();
            error = Api.AdkDeleteRecord(pData);
            if (error.lRc != Api.ADKE_OK)
            {
                String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                MessageBox.Show(errortext);
            }
            else
            {
                error = Api.AdkFirst(pData);
                if (error.lRc != Api.ADKE_OK)
                {
                    String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                    MessageBox.Show(errortext);
                }

                this.SetDataToFieldKund();

                if (bShowed)
                {
                    this.ShowKundList();
                }
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new About().ShowDialog();
        }

        private void ShowFktList()
        {
            FKT_LIST.Items.Clear();
            error = Api.AdkFirstEx(pData2, false); //Hmta utan rader
            if (error.lRc != Api.ADKE_OK)
            {
                String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                MessageBox.Show(errortext);
            }

            int iX = 0;
            while (error.lRc == Api.ADKE_OK)
            {
                error = Api.AdkGetNumeric(pData2, Api.ADK_OOI_HEAD_DOCUMENT_NUMBER, out Double FktNr);
                error = Api.AdkGetStr(pData2, Api.ADK_OOI_HEAD_CUSTOMER_NAME, out var FktKundNamn);
                error = Api.AdkGetDate(pData2, Api.ADK_OOI_HEAD_DOCUMENT_DATE1, out var InvoDate);
                error = Api.AdkGetNumeric(pData2, Api.ADK_OOI_HEAD_TOTAL_AMOUNT, out Double FktKr);
                error = Api.AdkGetDate(pData2, Api.ADK_OOI_HEAD_DOCUMENT_DATE2, out var PayDate);

                ListViewItem item = new ListViewItem(FktNr.ToString(), iX);
                item.SubItems.Add(FktKundNamn);
                item.SubItems.Add(InvoDate.HasValue ? InvoDate.Value.ToString("yyyy-MM-dd") : "");
                item.SubItems.Add(FktKr.ToString());
                item.SubItems.Add(PayDate.HasValue ? PayDate.Value.ToString("yyyy-MM-dd") : "");

                FKT_LIST.Items.Add(item);

                error = Api.AdkNextEx(pData2, false);
                iX++;
            }
        }

        private void KFKT_BN_LIST_Click(object sender, EventArgs e)
        {
            this.ShowFktList();
        }

        private void SetDataToFieldKFkt()
        {
            String info;
            error = Api.AdkGetStr(pData2, Api.ADK_OOI_HEAD_CUSTOMER_NUMBER, out info);
            KFKT_KUNDNR_BOX.Text = info;

            Api.AdkGetStr(pData2, Api.ADK_OOI_HEAD_CUSTOMER_NAME, out info);
            KFKT_KUNDNAMN_BOX.Text = info;

            Api.AdkGetStr(pData2, Api.ADK_OOI_HEAD_CUSTOMER_REFERENCE_NAME, out info);
            KFKT_ERREF_BOX.Text = info;

            Api.AdkGetNumeric(pData2, Api.ADK_OOI_HEAD_NROWS, out Double iXRows);


            FKT_ROWLIST.Items.Clear();
            for (int iX = 0; iX < iXRows; iX++)
            {
                error = Api.AdkGetRowData(pData2, iX, out var pRows);

                if (error.lRc != Api.ADKE_OK)
                {
                    String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                    MessageBox.Show(errortext);
                }

                Api.AdkGetStr(pRows, Api.ADK_OOI_ROW_ARTICLE_NUMBER, out info);
                ListViewItem item = new ListViewItem(info, iX);

                Api.AdkGetStr(pRows, Api.ADK_OOI_ROW_TEXT, out info);
                item.SubItems.Add(info);

                Double ant;
                Api.AdkGetNumeric(pRows, Api.ADK_OOI_ROW_QUANTITY2, out ant);
                item.SubItems.Add(ant.ToString());

                Api.AdkGetNumeric(pRows, Api.ADK_OOI_ROW_QUANTITY1, out ant);
                item.SubItems.Add(ant.ToString());
     
                FKT_ROWLIST.Items.Add(item);
            }
        }

        private void FKT_LIST_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (FKT_LIST.FocusedItem != null)
            {
                String doc = FKT_LIST.FocusedItem.Text;
                try
                {
                    Double.TryParse(doc, out var docnr);    
                    error = Api.AdkSetDouble(pData2, Api.ADK_OOI_HEAD_DOCUMENT_NUMBER, docnr);
                    error = Api.AdkFind2(pData2, true);
                    if (error.lRc != Api.ADKE_OK)
                    {
                        String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                        MessageBox.Show(errortext);
                    }
                    SetDataToFieldKFkt();
                    error = Api.AdkResetStruct(pData2);
                }
                catch
                {
                    MessageBox.Show("Problem med typkonvertering");
                }                
            }
        }

        private void KFKT_BN_NEW_Click(object sender, EventArgs e)
        {
            FKT_ROWLIST.Items.Clear();
            KFKT_ERREF_BOX.Text = "";
            KFKT_KUNDNR_BOX.Text = "";
            KFKT_KUNDNAMN_BOX.Text = "";

            NewCInv nyfaktura = new NewCInv();
            DialogResult dr = nyfaktura.ShowDialog(this);
            nyfaktura.Dispose();

            if(dr != DialogResult.Cancel)
                ShowFktList();
        }

        private void KFKT_BN_DEL_Click(object sender, EventArgs e)
        {
            if (FKT_LIST.FocusedItem != null)
            {
                String doc = FKT_LIST.FocusedItem.Text;

                try
                {
                    Double.TryParse(doc, out var docnr);

                    error = Api.AdkSetDouble(pData2, Api.ADK_OOI_HEAD_DOCUMENT_NUMBER, docnr);
                    error = Api.AdkDeleteRecord(pData2);
                    if (error.lRc != Api.ADKE_OK)
                    {
                        String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                        MessageBox.Show(errortext);
                    }
                }
                catch { }

                ShowFktList();
            }
            else
            {
                MessageBox.Show("Du mste markera en faktura");
            }
        }

        private void DemoProjektMainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (pData != 0)
            {
                error = Api.AdkDeleteStruct(pData);
                if (error.lRc != Api.ADKE_OK)
                {
                    String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                    MessageBox.Show(errortext);
                }
            }

            if (pData2 != 0)
            {
                error = Api.AdkDeleteStruct(pData2);
                if (error.lRc != Api.ADKE_OK)
                {
                    String errortext = Api.AdkGetErrorText(error, Api.ADK_ERROR_TEXT_TYPE.elRc);
                    MessageBox.Show(errortext);
                }
            }
        }
    }
}